/*
 * This file is part of Jelie,
 * (c) 2002 Julien Pilet <julien.pilet@epfl.ch> and
 * Stephane Magnenat <stephane.magnenat@epfl.ch>
 *
 * Jelie is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * Jelie is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*! \file
 * Parallel port JTAG connection.
 *
 * \author Julien Pilet <julien.pilet@epfl.ch>
 * \author Stephane Magnenat <stephane.magnenat@epfl.ch>
 */

#ifndef _PP_JTAG_H
#define _PP_JTAG_H

#include "jtag.h"

//! Parrallel port JTAG control class
class PPJTAG : public JTAGControl {
public:
	virtual bool init(int argc, char **argv);

	virtual void ireg(unsigned char *data);

	virtual void dreg(unsigned char *data, unsigned char length, bool writeOnly=false);

	virtual void jtagReset(void);

	virtual void cpuReset(bool state);

	virtual void trst(bool state);

	virtual void idle(unsigned int nbTCK);

	virtual void flushQueue(void) {}

	virtual const char *getName(void) const { return "pp"; }

	PPJTAG();

private:
	//! parrallel port number
	unsigned short port;
	unsigned char permBits, portVal;
	int jtagRestarted;

	//static unsigned char inb(unsigned short int port);
	//static void outb(unsigned char value, unsigned short int port);
	unsigned char readJTAG();
	void writeJTAG(unsigned char data);
	void jtagReg(unsigned char *_data, unsigned char length);
};

#endif // _PP_JTAG_H

